@@ -44,6 +44,8 @@ def test_embed():
         y0: int  # not closed over
         y1: int = y0 + y1  # used in `g`
         y2: int = y1 + 1  # only closed over by embed
+        h1 = None
+        h2 = None
         nonlocal x1
         nonlocal x2
 
@@ -54,17 +56,66 @@ def test_embed():
 
         x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
         print(x1, x2, y0, y1, y2)
-        # passing the enclosing function allows variables from the parent scopes
-        # which were closed over to be accessed and modified (x and y)
+        # passing the enclosing function allows the cells of the variables from
+        # the parent scopes which were closed over to be used for closures, i.e.
+        # new functions using these variables in the shell will modify the same
+        # variables as any existing functions or future functions
         # note that none of the shell will see `x0`
         embed(f)
-        # run: x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]; g()
+        # run:
+        """
+        def h1():
+            nonlocal x1, x2, y0, y1, y2
+            print("in h1", x1, x2, y0, y1, y2)
+            x1, x2, y0, y1, y2 = [x + 10 for x in [x1, x2, y0, y1, y2]]
+        x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
+        print("before g", x1, x2, y0, y1, y2)
+        g()
+        print("after g", x1, x2, y0, y1, y2)
+        h1()
+        print("after h1", x1, x2, y0, y1, y2)
+        """
         x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
         print(x1, x2, y0, y1, y2)
-        # passing a closure over local variables allow the specified variables
-        # to be accessed and modified (y1 and y2)
+        # you can also pass more than one function to enable specified local
+        # variables to be properly closed over (y1 and y2); enclosing them in a
+        # lambda ensures that they become cell variables
         embed([f, lambda: (y1, y2)])
-        # run: x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]; g()
+        # run:
+        """
+        def h2():
+            nonlocal x1, x2, y0, y1, y2
+            print("in h2", x1, x2, y0, y1, y2)
+            x1, x2, y0, y1, y2 = [x + 10 for x in [x1, x2, y0, y1, y2]]
+        x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
+        print("before g", x1, x2, y0, y1, y2)
+        g()
+        print("after g", x1, x2, y0, y1, y2)
+        h1()
+        print("after h1", x1, x2, y0, y1, y2)
+        h2()
+        print("after h2", x1, x2, y0, y1, y2)
+        """
+        x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
+        print(x1, x2, y0, y1, y2)
+        embed([f, lambda: (y1, y2)])
+        # run:
+        """
+        def h3():
+            nonlocal x1, x2, y0, y1, y2
+            print("in h3", x1, x2, y0, y1, y2)
+            x1, x2, y0, y1, y2 = [x + 10 for x in [x1, x2, y0, y1, y2]]
+        x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
+        print("before g", x1, x2, y0, y1, y2)
+        g()
+        print("after g", x1, x2, y0, y1, y2)
+        h1()
+        print("after h1", x1, x2, y0, y1, y2)
+        h2()
+        print("after h2", x1, x2, y0, y1, y2)
+        h3()
+        print("after h3", x1, x2, y0, y1, y2)
+        """
         x1, x2, y0, y1, y2 = [x + 100 for x in [x1, x2, y0, y1, y2]]
         print(x1, x2, y0, y1, y2)
 
@@ -79,6 +130,8 @@ def test_embed2():
     x2 = 2  # used in `f` and `g`
 
     def f(y0, y1):
+        h1 = None
+        h2 = None
         y0: int  # not closed over
         y1: int = y0 + y1  # used in `g`
         nonlocal x1
@@ -92,13 +145,40 @@ def test_embed2():
         x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]
         print(x1, x2, y0, y1)
         # embed2 uses a mocked up globals object which diverges from the real
-        # globals/locals and will never be able to persist any changes
+        # globals/locals and will only be able to persist final values of locals
+        # and the closures will not be closed over the correct cell variables
         embed2()
-        # run: x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]; g()
+        # run:
+        """
+        def h1():
+            global x1, x2, y0, y1
+            print("in h1", x1, x2, y0, y1)
+            x1, x2, y0, y1 = [x + 10 for x in [x1, x2, y0, y1]]
+        x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]
+        print("before g", x1, x2, y0, y1)
+        g()
+        print("after g", x1, x2, y0, y1)
+        h1()
+        print("after h1", x1, x2, y0, y1)
+        """
         x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]
         print(x1, x2, y0, y1)
         embed2()
-        # run: x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]; g()
+        # run:
+        """
+        def h2():
+            global x1, x2, y0, y1
+            print("in h2", x1, x2, y0, y1)
+            x1, x2, y0, y1 = [x + 10 for x in [x1, x2, y0, y1]]
+        x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]
+        print("before g", x1, x2, y0, y1)
+        g()
+        print("after g", x1, x2, y0, y1)
+        h1()
+        print("after h1", x1, x2, y0, y1)
+        h2()
+        print("after h2", x1, x2, y0, y1)
+        """
         x1, x2, y0, y1 = [x + 100 for x in [x1, x2, y0, y1]]
         print(x1, x2, y0, y1)
 
